'use strict';

const JSON2CSVParser = require('json2csv/lib/JSON2CSVParser');
const JSON2CSVAsyncParser = require('json2csv/lib/JSON2CSVAsyncParser');
const JSON2CSVStreamParser = require('json2csv/lib/JSON2CSVStreamParser');
const JSON2CSVTransform = require('json2csv/lib/JSON2CSVTransform');

// Transforms
const flatten = require('json2csv/lib/transforms/flatten');
const unwind = require('json2csv/lib/transforms/unwind');

// Formatters
const defaultFormatter = require('json2csv/lib/formatters/default');
const number = require('json2csv/lib/formatters/number');
const string = require('json2csv/lib/formatters/string');
const stringQuoteOnlyIfNecessary =  require('json2csv/lib/formatters/stringQuoteOnlyIfNecessary');
const stringExcel = require('json2csv/lib/formatters/stringExcel');
const symbol = require('json2csv/lib/formatters/symbol');
const object = require('json2csv/lib/formatters/object');

module.exports.Parser = JSON2CSVParser;
module.exports.AsyncParser = JSON2CSVAsyncParser;
module.exports.StreamParser = JSON2CSVStreamParser;
module.exports.Transform = JSON2CSVTransform;

// Convenience method to keep the API similar to version 3.X
module.exports.parse = (data, opts) => new JSON2CSVParser(opts).parse(data);
module.exports.parseAsync = (data, opts, transformOpts) => new JSON2CSVAsyncParser(opts, transformOpts).parse(data).promise();

module.exports.transforms = {
  flatten,
  unwind,
};

module.exports.formatters = {
  default: defaultFormatter,
  number,
  string,
  stringQuoteOnlyIfNecessary,
  stringExcel,
  symbol,
  object,
};
